<?php
// Do not allow directly accessing this file.
if (!defined('ABSPATH')) {
    exit('Direct script access denied.');
}
if (!class_exists('G5Shop_Widget_Price_Filter')) {
    class G5Shop_Widget_Price_Filter extends GSF_Widget {

        public function __construct()
        {
            $this->widget_cssclass = 'g5shop__widget-price-filter';
            $this->widget_id = 'g5shop__widget_price_filter';
            $this->widget_name = esc_html__('G5Plus: Product Price Filter', 'g5-shop');
            $this->widget_description = esc_html__( 'Display a product price filter list', 'g5-shop' );
            $this->settings = array(
                'fields' => array(
                    array(
                        'id'      => 'title',
                        'title'   => esc_html__('Title', 'g5-shop'),
                        'type'    => 'text',
                        'default' => esc_html__( 'Filter by price', 'g5-shop' ),
                    ),
                    array(
                        'id' => 'max_price_ranges',
                        'title' => __('Max price ranges', 'g5-shop'),
                        'type'       => 'text',
                        'input_type' => 'number',
                        'args' => array(
                            'min' => '1',
                            'max' => '20',
                            'step' => '1'
                        ),
                        'default' => 5,
                    ),
                )
            );
            parent::__construct();
        }

        function widget($args, $instance)
        {
	        // Requires lookup table added in 3.6.
	        if ( version_compare( get_option( 'woocommerce_db_version', null ), '3.6', '<' ) ) {
		        return;
	        }

	        if ( ! is_shop() && ! is_product_taxonomy() ) {
		        return;
	        }

	        // If there are not posts and we're not filtering, hide the widget.
	        if ( ! WC()->query->get_main_query()->post_count && ! isset( $_GET['min_price'] ) && ! isset( $_GET['max_price'] ) ) { // WPCS: input var ok, CSRF ok.
		        return;
	        }

            if ($this->get_cached_widget($instance)) {
                return;
            }


            extract($args, EXTR_SKIP);

	        $step = (!empty($instance['max_price_ranges'])) ? intval($instance['max_price_ranges']) : 5;


	        // Find min and max price in current result set.
	        $prices    = $this->get_filtered_price();
	        $min_price = $prices->min_price;
	        $max_price = $prices->max_price;

	        // Check to see if we should add taxes to the prices if store are excl tax but display incl.
	        $tax_display_mode = get_option( 'woocommerce_tax_display_shop' );

	        if ( wc_tax_enabled() && ! wc_prices_include_tax() && 'incl' === $tax_display_mode ) {
		        $tax_class = apply_filters( 'woocommerce_price_filter_widget_tax_class', '' ); // Uses standard tax class.
		        $tax_rates = WC_Tax::get_rates( $tax_class );

		        if ( $tax_rates ) {
			        $min_price += WC_Tax::get_tax_total( WC_Tax::calc_exclusive_tax( $min_price, $tax_rates ) );
			        $max_price += WC_Tax::get_tax_total( WC_Tax::calc_exclusive_tax( $max_price, $tax_rates ) );
		        }
	        }

	        $min_price = apply_filters( 'woocommerce_price_filter_widget_min_amount', floor( $min_price / $step ) * $step );
	        $max_price = apply_filters( 'woocommerce_price_filter_widget_max_amount', ceil( $max_price / $step ) * $step );

	        // If both min and max are equal, we don't need a slider.
	        if ( $min_price === $max_price ) {
		        return;
	        }

	        $current_min_price = isset( $_GET['min_price'] ) ? floor( floatval( wp_unslash( $_GET['min_price'] ) ) / $step ) * $step : $min_price; // WPCS: input var ok, CSRF ok.
	        $current_max_price = isset( $_GET['max_price'] ) ? ceil( floatval( wp_unslash( $_GET['max_price'] ) ) / $step ) * $step : $max_price; // WPCS: input var ok, CSRF ok.

            $links = $this->generate_price_links($max_price, $current_min_price, $current_max_price,$step );
            if (empty($links)) return;

            ob_start();
            $this->widget_start($args,$instance);
            ?>
                <ul class="g5shop__price-filter">
                    <?php foreach ($links as $link): ?>
                        <li class="<?php echo esc_attr($link['class'])?>">
                            <a href="<?php echo esc_url($link['href'])?>"><?php echo ($link['title'])?></a>
                        </li>
                    <?php endforeach; ?>
                </ul>
            <?php
            $this->widget_end($args);
            echo $this->cache_widget( $args, ob_get_clean() ); // WPCS: XSS ok.
        }

	    /**
	     * Get filtered min price for current products.
	     *
	     * @return int
	     */
	    protected function get_filtered_price() {
		    global $wpdb;

		    $args       = WC()->query->get_main_query()->query_vars;
		    $tax_query  = isset( $args['tax_query'] ) ? $args['tax_query'] : array();
		    $meta_query = isset( $args['meta_query'] ) ? $args['meta_query'] : array();

		    if ( ! is_post_type_archive( 'product' ) && ! empty( $args['taxonomy'] ) && ! empty( $args['term'] ) ) {
			    $tax_query[] = WC()->query->get_main_tax_query();
		    }

		    foreach ( $meta_query + $tax_query as $key => $query ) {
			    if ( ! empty( $query['price_filter'] ) || ! empty( $query['rating_filter'] ) ) {
				    unset( $meta_query[ $key ] );
			    }
		    }

		    $meta_query = new WP_Meta_Query( $meta_query );
		    $tax_query  = new WP_Tax_Query( $tax_query );
		    $search     = WC_Query::get_main_search_query_sql();

		    $meta_query_sql   = $meta_query->get_sql( 'post', $wpdb->posts, 'ID' );
		    $tax_query_sql    = $tax_query->get_sql( $wpdb->posts, 'ID' );
		    $search_query_sql = $search ? ' AND ' . $search : '';

		    $sql = "
			SELECT min( min_price ) as min_price, MAX( max_price ) as max_price
			FROM {$wpdb->wc_product_meta_lookup}
			WHERE product_id IN (
				SELECT ID FROM {$wpdb->posts}
				" . $tax_query_sql['join'] . $meta_query_sql['join'] . "
				WHERE {$wpdb->posts}.post_type IN ('" . implode( "','", array_map( 'esc_sql', apply_filters( 'woocommerce_price_filter_post_type', array( 'product' ) ) ) ) . "')
				AND {$wpdb->posts}.post_status = 'publish'
				" . $tax_query_sql['where'] . $meta_query_sql['where'] . $search_query_sql . '
			)';

		    $sql = apply_filters( 'woocommerce_price_filter_sql', $sql, $meta_query_sql, $tax_query_sql );

		    return $wpdb->get_row( $sql ); // WPCS: unprepared SQL ok.
	    }

        private function generate_price_links( $max, $min_price, $max_price ,$steps) {
            $links = array();

            // Remember current filters/search
            $link          = g5shop_widget_get_current_page_url();
            $link_no_price = remove_query_arg( 'min_price', $link );
            $link_no_price = remove_query_arg( 'max_price', $link_no_price );

            $need_more = false;

            $step_value = $max / $steps;

            if ( $step_value < 10 ) {
                $step_value = 10;
            }

            $step_value = round( $step_value, - 1 );

            // Link to all prices
            $links[] = array(
                'href'  => $link_no_price,
                'title' => esc_html__( 'All', 'g5-shop' ),
                'class' => '',
            );

            for ( $i = 0; $i < (int) $steps; $i ++ ) {

                $step_class = $href = '';

                $step_min = $step_value * $i;

                $step_max = $step_value * ( $i + 1 );

                if ( $step_max > $max ) {
                    $need_more = true;
                    $i ++;
                    break;
                }

                $href = add_query_arg( 'min_price', $step_min, $link );
                $href = add_query_arg( 'max_price', $step_max, $href );

                $step_title = wc_price( $step_min ) . ' - ' . wc_price( $step_max );

                if ( ! empty( $min_price ) && ! empty( $max_price ) && ( $min_price >= $step_min && $max_price <= $step_max ) || ( $i == 0 && ! empty( $max_price ) && $max_price <= $step_max ) ) {
                    $step_class = 'current';
                }

                $links[] = array(
                    'href'  => $href,
                    'title' => $step_title,
                    'class' => $step_class,
                );
            }

            if ( $max > $step_max ) {
                $need_more = true;
                $step_min  = $step_value * $i;
            }

            if ( $need_more ) {

                $step_class = $href = '';

                $href = add_query_arg( 'min_price', $step_min, $link );
                $href = add_query_arg( 'max_price', $max, $href );

                $step_title = wc_price( $step_min ) . ' +';

                if ( $min_price >= $step_min && $max_price <= $max ) {
                    $step_class = 'current-state';
                }

                $links[] = array(
                    'href'  => $href,
                    'title' => $step_title,
                    'class' => $step_class,
                );
            }

            return $links;
        }
    }
}